from collections import defaultdict
from skidl import Pin, Part, Alias, SchLib, SKIDL, TEMPLATE

from skidl.pin import pin_types

SKIDL_lib_version = '0.0.1'

Interface_LineDriver = SchLib(tool=SKIDL).add_parts(*[
        Part(**{ 'name':'DS7820', 'dest':TEMPLATE, 'tool':SKIDL, 'aliases':Alias({'DS7820'}), 'ref_prefix':'U', 'fplist':['Package_DIP:DIP-14_W7.62mm'], 'footprint':'Package_DIP:DIP-14_W7.62mm', 'keywords':'Dual line receiver', 'description':'', 'datasheet':'http://pdf1.alldatasheet.com/datasheet-pdf/view/942427/NSC/DS7820.html', 'search_text':'/usr/share/kicad/symbols/Interface_LineDriver.kicad_sym\nDS7820\n\nDual line receiver', 'pins':[
            Pin(num='1',name='INPUT',func=pin_types.INPUT,unit=1),
            Pin(num='10',name='STROBE',func=pin_types.INPUT,unit=1),
            Pin(num='11',name='INPUT',func=pin_types.INPUT,unit=1),
            Pin(num='12',name='TERMINATION',func=pin_types.INPUT,unit=1),
            Pin(num='13',name='INPUT',func=pin_types.INPUT,unit=1),
            Pin(num='14',name='VCC',func=pin_types.PWRIN,unit=1),
            Pin(num='2',name='TERMINATION',func=pin_types.INPUT,unit=1),
            Pin(num='3',name='INPUT',func=pin_types.INPUT,unit=1),
            Pin(num='4',name='STROBE',func=pin_types.INPUT,unit=1),
            Pin(num='5',name='RESPONSE_TIME',func=pin_types.INPUT,unit=1),
            Pin(num='6',name='OUTPUT',func=pin_types.INPUT,unit=1),
            Pin(num='7',name='GND',func=pin_types.PWRIN,unit=1),
            Pin(num='8',name='OUTPUT',func=pin_types.OUTPUT,unit=1),
            Pin(num='9',name='RESPONSE_TIME',func=pin_types.INPUT,unit=1)], 'unit_defs':[] }),
        Part(**{ 'name':'DS7830', 'dest':TEMPLATE, 'tool':SKIDL, 'aliases':Alias({'DS7830'}), 'ref_prefix':'U', 'fplist':['Package_DIP:DIP-14_W7.62mm'], 'footprint':'Package_DIP:DIP-14_W7.62mm', 'keywords':'Dual differential line driver', 'description':'', 'datasheet':'http://pdf1.alldatasheet.com/datasheet-pdf/view/8473/NSC/DS7830J.html', 'search_text':'/usr/share/kicad/symbols/Interface_LineDriver.kicad_sym\nDS7830\n\nDual differential line driver', 'pins':[
            Pin(num='1',name='A_1',func=pin_types.INPUT,unit=1),
            Pin(num='10',name='B_4',func=pin_types.INPUT,unit=1),
            Pin(num='11',name='B_3',func=pin_types.INPUT,unit=1),
            Pin(num='12',name='B_2',func=pin_types.INPUT,unit=1),
            Pin(num='13',name='B_1',func=pin_types.INPUT,unit=1),
            Pin(num='14',name='VCC',func=pin_types.PWRIN,unit=1),
            Pin(num='2',name='A_2',func=pin_types.INPUT,unit=1),
            Pin(num='3',name='A_3',func=pin_types.INPUT,unit=1),
            Pin(num='4',name='A_4',func=pin_types.INPUT,unit=1),
            Pin(num='5',name='A_AND_OUTPUT',func=pin_types.INPUT,unit=1),
            Pin(num='6',name='A_NAND_OUTPUT',func=pin_types.INPUT,unit=1),
            Pin(num='7',name='GND',func=pin_types.PWRIN,unit=1),
            Pin(num='8',name='B_NAND_OUTPUT',func=pin_types.INPUT,unit=1),
            Pin(num='9',name='B_AND_OUTPUT',func=pin_types.INPUT,unit=1)], 'unit_defs':[] }),
        Part(**{ 'name':'DS89C21', 'dest':TEMPLATE, 'tool':SKIDL, 'aliases':Alias({'DS89C21'}), 'ref_prefix':'U', 'fplist':['Package_SO:SOIC-8_3.9x4.9mm_P1.27mm'], 'footprint':'Package_SO:SOIC-8_3.9x4.9mm_P1.27mm', 'keywords':'rs422 tranciever', 'description':'', 'datasheet':'http://www.ti.com/lit/ds/symlink/ds89c21.pdf', 'search_text':'/usr/share/kicad/symbols/Interface_LineDriver.kicad_sym\nDS89C21\n\nrs422 tranciever', 'pins':[
            Pin(num='1',name='VCC',func=pin_types.PWRIN,unit=1),
            Pin(num='2',name='RO',func=pin_types.OUTPUT,unit=1),
            Pin(num='3',name='DI',func=pin_types.INPUT,unit=1),
            Pin(num='4',name='GND',func=pin_types.PWRIN,unit=1),
            Pin(num='5',name='~{DO}',func=pin_types.OUTPUT,unit=1),
            Pin(num='6',name='DO',func=pin_types.OUTPUT,unit=1),
            Pin(num='7',name='~{RI}',func=pin_types.INPUT,unit=1),
            Pin(num='8',name='RI',func=pin_types.INPUT,unit=1)], 'unit_defs':[] }),
        Part(**{ 'name':'EL7242C', 'dest':TEMPLATE, 'tool':SKIDL, 'aliases':Alias({'EL7242C'}), 'ref_prefix':'U', 'fplist':['Package_DIP:DIP-8_W7.62mm'], 'footprint':'Package_DIP:DIP-8_W7.62mm', 'keywords':'Translateur TTL->VMOS. Double driver VMOS', 'description':'', 'datasheet':'http://pdf.datasheetcatalog.com/datasheet/elantec/EL7242CN.pdf', 'search_text':'/usr/share/kicad/symbols/Interface_LineDriver.kicad_sym\nEL7242C\n\nTranslateur TTL->VMOS. Double driver VMOS', 'pins':[
            Pin(num='5',name='GND',func=pin_types.PWRIN),
            Pin(num='8',name='V+',func=pin_types.PWRIN),
            Pin(num='1',name='IN1',func=pin_types.INPUT,unit=1),
            Pin(num='2',name='IN',func=pin_types.INPUT,unit=1),
            Pin(num='7',name='OUT',func=pin_types.OUTPUT,unit=1),
            Pin(num='3',name='IN1',func=pin_types.INPUT,unit=2),
            Pin(num='4',name='IN',func=pin_types.INPUT,unit=2),
            Pin(num='6',name='OUT',func=pin_types.OUTPUT,unit=2)], 'unit_defs':[{'label': 'uA', 'num': 1, 'pin_nums': ['5', '8', '2', '7', '1']},{'label': 'uB', 'num': 2, 'pin_nums': ['5', '8', '6', '4', '3']}] }),
        Part(**{ 'name':'MC3486N', 'dest':TEMPLATE, 'tool':SKIDL, 'aliases':Alias({'MC3486N'}), 'ref_prefix':'U', 'fplist':['Package_DIP:DIP-14_W7.62mm'], 'footprint':'Package_DIP:DIP-14_W7.62mm', 'keywords':'Quadruple differential line receiver', 'description':'', 'datasheet':'http://www.ti.com/lit/ds/symlink/mc3486.pdf', 'search_text':'/usr/share/kicad/symbols/Interface_LineDriver.kicad_sym\nMC3486N\n\nQuadruple differential line receiver', 'pins':[
            Pin(num='1',name='E-',func=pin_types.INPUT,unit=1),
            Pin(num='16',name='VCC',func=pin_types.PWRIN,unit=1),
            Pin(num='2',name='E+',func=pin_types.INPUT,unit=1),
            Pin(num='3',name='OUT',func=pin_types.TRISTATE,unit=1),
            Pin(num='4',name='ENABLE',func=pin_types.INPUT,unit=1),
            Pin(num='8',name='GND',func=pin_types.PWRIN,unit=1),
            Pin(num='16',name='VCC',func=pin_types.PWRIN,unit=2),
            Pin(num='4',name='ENABLE',func=pin_types.INPUT,unit=2),
            Pin(num='5',name='OUT',func=pin_types.TRISTATE,unit=2),
            Pin(num='6',name='E+',func=pin_types.INPUT,unit=2),
            Pin(num='7',name='E-',func=pin_types.INPUT,unit=2),
            Pin(num='8',name='GND',func=pin_types.PWRIN,unit=2),
            Pin(num='10',name='E+',func=pin_types.INPUT,unit=3),
            Pin(num='11',name='OUT',func=pin_types.TRISTATE,unit=3),
            Pin(num='12',name='ENABLE',func=pin_types.INPUT,unit=3),
            Pin(num='16',name='VCC',func=pin_types.PWRIN,unit=3),
            Pin(num='8',name='GND',func=pin_types.PWRIN,unit=3),
            Pin(num='9',name='E-',func=pin_types.INPUT,unit=3),
            Pin(num='12',name='ENABLE',func=pin_types.INPUT,unit=4),
            Pin(num='13',name='OUT',func=pin_types.TRISTATE,unit=4),
            Pin(num='14',name='E+',func=pin_types.INPUT,unit=4),
            Pin(num='15',name='E-',func=pin_types.INPUT,unit=4),
            Pin(num='16',name='VCC',func=pin_types.PWRIN,unit=4),
            Pin(num='8',name='GND',func=pin_types.PWRIN,unit=4)], 'unit_defs':[{'label': 'uA', 'num': 1, 'pin_nums': ['8', '16', '3', '2', '4', '1']},{'label': 'uB', 'num': 2, 'pin_nums': ['4', '7', '5', '6', '16', '8']},{'label': 'uC', 'num': 3, 'pin_nums': ['8', '10', '12', '11', '16', '9']},{'label': 'uD', 'num': 4, 'pin_nums': ['8', '15', '13', '12', '16', '14']}] }),
        Part(**{ 'name':'MC3487DX', 'dest':TEMPLATE, 'tool':SKIDL, 'aliases':Alias({'MC3487DX'}), 'ref_prefix':'U', 'fplist':['Package_SO:SOIC-16_3.9x9.9mm_P1.27mm'], 'footprint':'Package_SO:SOIC-16_3.9x9.9mm_P1.27mm', 'keywords':'Four independent differential line drivers', 'description':'', 'datasheet':'http://www.ti.com/lit/ds/symlink/mc3487.pdf', 'search_text':'/usr/share/kicad/symbols/Interface_LineDriver.kicad_sym\nMC3487DX\n\nFour independent differential line drivers', 'pins':[
            Pin(num='1',name='INPUT',func=pin_types.INPUT,unit=1),
            Pin(num='16',name='VCC',func=pin_types.PWRIN,unit=1),
            Pin(num='2',name='OUT+',func=pin_types.TRISTATE,unit=1),
            Pin(num='3',name='OUT-',func=pin_types.TRISTATE,unit=1),
            Pin(num='4',name='ENABLE',func=pin_types.INPUT,unit=1),
            Pin(num='8',name='GND',func=pin_types.PWRIN,unit=1),
            Pin(num='16',name='VCC',func=pin_types.PWRIN,unit=2),
            Pin(num='4',name='ENABLE',func=pin_types.INPUT,unit=2),
            Pin(num='5',name='OUT-',func=pin_types.TRISTATE,unit=2),
            Pin(num='6',name='OUT+',func=pin_types.TRISTATE,unit=2),
            Pin(num='7',name='INPUT',func=pin_types.INPUT,unit=2),
            Pin(num='8',name='GND',func=pin_types.PWRIN,unit=2),
            Pin(num='10',name='OUT+',func=pin_types.TRISTATE,unit=3),
            Pin(num='11',name='OUT-',func=pin_types.TRISTATE,unit=3),
            Pin(num='12',name='ENABLE',func=pin_types.INPUT,unit=3),
            Pin(num='16',name='VCC',func=pin_types.PWRIN,unit=3),
            Pin(num='8',name='GND',func=pin_types.PWRIN,unit=3),
            Pin(num='9',name='INPUT',func=pin_types.INPUT,unit=3),
            Pin(num='12',name='ENABLE',func=pin_types.INPUT,unit=4),
            Pin(num='13',name='OUT-',func=pin_types.TRISTATE,unit=4),
            Pin(num='14',name='OUT+',func=pin_types.TRISTATE,unit=4),
            Pin(num='15',name='INPUT',func=pin_types.INPUT,unit=4),
            Pin(num='16',name='VCC',func=pin_types.PWRIN,unit=4),
            Pin(num='8',name='GND',func=pin_types.PWRIN,unit=4)], 'unit_defs':[{'label': 'uA', 'num': 1, 'pin_nums': ['8', '1', '3', '16', '4', '2']},{'label': 'uB', 'num': 2, 'pin_nums': ['7', '5', '8', '16', '6', '4']},{'label': 'uC', 'num': 3, 'pin_nums': ['10', '12', '8', '16', '11', '9']},{'label': 'uD', 'num': 4, 'pin_nums': ['12', '16', '14', '15', '13', '8']}] }),
        Part(**{ 'name':'MC3487N', 'dest':TEMPLATE, 'tool':SKIDL, 'aliases':Alias({'MC3487N'}), 'ref_prefix':'U', 'fplist':['Package_DIP:DIP-16_W7.62mm'], 'footprint':'Package_DIP:DIP-16_W7.62mm', 'keywords':'Four independent differential line drivers', 'description':'', 'datasheet':'http://www.ti.com/lit/ds/symlink/mc3487.pdf', 'search_text':'/usr/share/kicad/symbols/Interface_LineDriver.kicad_sym\nMC3487N\n\nFour independent differential line drivers', 'pins':[
            Pin(num='1',name='INPUT',func=pin_types.INPUT,unit=1),
            Pin(num='16',name='VCC',func=pin_types.PWRIN,unit=1),
            Pin(num='2',name='OUT+',func=pin_types.TRISTATE,unit=1),
            Pin(num='3',name='OUT-',func=pin_types.TRISTATE,unit=1),
            Pin(num='4',name='ENABLE',func=pin_types.INPUT,unit=1),
            Pin(num='8',name='GND',func=pin_types.PWRIN,unit=1),
            Pin(num='16',name='VCC',func=pin_types.PWRIN,unit=2),
            Pin(num='4',name='ENABLE',func=pin_types.INPUT,unit=2),
            Pin(num='5',name='OUT-',func=pin_types.TRISTATE,unit=2),
            Pin(num='6',name='OUT+',func=pin_types.TRISTATE,unit=2),
            Pin(num='7',name='INPUT',func=pin_types.INPUT,unit=2),
            Pin(num='8',name='GND',func=pin_types.PWRIN,unit=2),
            Pin(num='10',name='OUT+',func=pin_types.TRISTATE,unit=3),
            Pin(num='11',name='OUT-',func=pin_types.TRISTATE,unit=3),
            Pin(num='12',name='ENABLE',func=pin_types.INPUT,unit=3),
            Pin(num='16',name='VCC',func=pin_types.PWRIN,unit=3),
            Pin(num='8',name='GND',func=pin_types.PWRIN,unit=3),
            Pin(num='9',name='INPUT',func=pin_types.INPUT,unit=3),
            Pin(num='12',name='ENABLE',func=pin_types.INPUT,unit=4),
            Pin(num='13',name='OUT-',func=pin_types.TRISTATE,unit=4),
            Pin(num='14',name='OUT+',func=pin_types.TRISTATE,unit=4),
            Pin(num='15',name='INPUT',func=pin_types.INPUT,unit=4),
            Pin(num='16',name='VCC',func=pin_types.PWRIN,unit=4),
            Pin(num='8',name='GND',func=pin_types.PWRIN,unit=4)], 'unit_defs':[{'label': 'uA', 'num': 1, 'pin_nums': ['4', '1', '2', '16', '8', '3']},{'label': 'uB', 'num': 2, 'pin_nums': ['16', '6', '4', '7', '5', '8']},{'label': 'uC', 'num': 3, 'pin_nums': ['11', '16', '9', '10', '8', '12']},{'label': 'uD', 'num': 4, 'pin_nums': ['8', '15', '13', '12', '14', '16']}] }),
        Part(**{ 'name':'UA9637', 'dest':TEMPLATE, 'tool':SKIDL, 'aliases':Alias({'UA9637'}), 'ref_prefix':'U', 'fplist':['Package_DIP:DIP-8_W7.62mm'], 'footprint':'Package_DIP:DIP-8_W7.62mm', 'keywords':'Dual differential line receiver', 'description':'', 'datasheet':'http://pdf.datasheetcatalog.com/datasheets2/28/284473_1.pdf', 'search_text':'/usr/share/kicad/symbols/Interface_LineDriver.kicad_sym\nUA9637\n\nDual differential line receiver', 'pins':[
            Pin(num='1',name='VCC',func=pin_types.PWRIN,unit=1),
            Pin(num='2',name='OUT',func=pin_types.OUTPUT,unit=1),
            Pin(num='4',name='GND',func=pin_types.PWRIN,unit=1),
            Pin(num='7',name='IN-',func=pin_types.INPUT,unit=1),
            Pin(num='8',name='IN+',func=pin_types.INPUT,unit=1),
            Pin(num='1',name='VCC',func=pin_types.PWRIN,unit=2),
            Pin(num='3',name='OUT',func=pin_types.OUTPUT,unit=2),
            Pin(num='4',name='GND',func=pin_types.PWRIN,unit=2),
            Pin(num='5',name='IN-',func=pin_types.INPUT,unit=2),
            Pin(num='6',name='IN+',func=pin_types.INPUT,unit=2)], 'unit_defs':[{'label': 'uA', 'num': 1, 'pin_nums': ['7', '8', '1', '4', '2']},{'label': 'uB', 'num': 2, 'pin_nums': ['1', '5', '3', '6', '4']}] }),
        Part(**{ 'name':'UA9638CD', 'dest':TEMPLATE, 'tool':SKIDL, 'aliases':Alias({'UA9638CD'}), 'ref_prefix':'U', 'fplist':['Package_SO:SOIC-8_3.9x4.9mm_P1.27mm'], 'footprint':'Package_SO:SOIC-8_3.9x4.9mm_P1.27mm', 'keywords':'Dual high-speed differential line driver', 'description':'', 'datasheet':'http://www.ti.com/lit/ds/symlink/ua9638.pdf', 'search_text':'/usr/share/kicad/symbols/Interface_LineDriver.kicad_sym\nUA9638CD\n\nDual high-speed differential line driver', 'pins':[
            Pin(num='1',name='VCC',func=pin_types.PWRIN,unit=1),
            Pin(num='2',name='IN',func=pin_types.INPUT,unit=1),
            Pin(num='4',name='GND',func=pin_types.PWRIN,unit=1),
            Pin(num='7',name='OUTA',func=pin_types.OUTPUT,unit=1),
            Pin(num='8',name='OUTB',func=pin_types.OUTPUT,unit=1),
            Pin(num='1',name='VCC',func=pin_types.PWRIN,unit=2),
            Pin(num='3',name='IN',func=pin_types.INPUT,unit=2),
            Pin(num='4',name='GND',func=pin_types.PWRIN,unit=2),
            Pin(num='5',name='OUTA',func=pin_types.OUTPUT,unit=2),
            Pin(num='6',name='OUTB',func=pin_types.OUTPUT,unit=2)], 'unit_defs':[{'label': 'uA', 'num': 1, 'pin_nums': ['1', '2', '7', '4', '8']},{'label': 'uB', 'num': 2, 'pin_nums': ['3', '5', '6', '4', '1']}] }),
        Part(**{ 'name':'UA9638CP', 'dest':TEMPLATE, 'tool':SKIDL, 'aliases':Alias({'UA9638CP'}), 'ref_prefix':'U', 'fplist':['Package_DIP:DIP-8_W7.62mm'], 'footprint':'Package_DIP:DIP-8_W7.62mm', 'keywords':'Dual high-speed differential line driver', 'description':'', 'datasheet':'http://www.ti.com/lit/ds/symlink/ua9638.pdf', 'search_text':'/usr/share/kicad/symbols/Interface_LineDriver.kicad_sym\nUA9638CP\n\nDual high-speed differential line driver', 'pins':[
            Pin(num='1',name='VCC',func=pin_types.PWRIN,unit=1),
            Pin(num='2',name='IN',func=pin_types.INPUT,unit=1),
            Pin(num='4',name='GND',func=pin_types.PWRIN,unit=1),
            Pin(num='7',name='OUTA',func=pin_types.OUTPUT,unit=1),
            Pin(num='8',name='OUTB',func=pin_types.OUTPUT,unit=1),
            Pin(num='1',name='VCC',func=pin_types.PWRIN,unit=2),
            Pin(num='3',name='IN',func=pin_types.INPUT,unit=2),
            Pin(num='4',name='GND',func=pin_types.PWRIN,unit=2),
            Pin(num='5',name='OUTA',func=pin_types.OUTPUT,unit=2),
            Pin(num='6',name='OUTB',func=pin_types.OUTPUT,unit=2)], 'unit_defs':[{'label': 'uA', 'num': 1, 'pin_nums': ['4', '8', '7', '2', '1']},{'label': 'uB', 'num': 2, 'pin_nums': ['3', '5', '6', '4', '1']}] }),
        Part(**{ 'name':'DS8830', 'dest':TEMPLATE, 'tool':SKIDL, 'aliases':Alias({'DS8830'}), 'ref_prefix':'U', 'fplist':['Package_DIP:DIP-14_W7.62mm', 'Package_DIP:DIP-14_W7.62mm'], 'footprint':'Package_DIP:DIP-14_W7.62mm', 'keywords':'Dual differential line driver', 'description':'', 'datasheet':'http://pdf1.alldatasheet.com/datasheet-pdf/view/8473/NSC/DS7830J.html', 'search_text':'/usr/share/kicad/symbols/Interface_LineDriver.kicad_sym\nDS8830\n\nDual differential line driver', 'pins':[
            Pin(num='1',name='A_1',func=pin_types.INPUT,unit=1),
            Pin(num='10',name='B_4',func=pin_types.INPUT,unit=1),
            Pin(num='11',name='B_3',func=pin_types.INPUT,unit=1),
            Pin(num='12',name='B_2',func=pin_types.INPUT,unit=1),
            Pin(num='13',name='B_1',func=pin_types.INPUT,unit=1),
            Pin(num='14',name='VCC',func=pin_types.PWRIN,unit=1),
            Pin(num='2',name='A_2',func=pin_types.INPUT,unit=1),
            Pin(num='3',name='A_3',func=pin_types.INPUT,unit=1),
            Pin(num='4',name='A_4',func=pin_types.INPUT,unit=1),
            Pin(num='5',name='A_AND_OUTPUT',func=pin_types.INPUT,unit=1),
            Pin(num='6',name='A_NAND_OUTPUT',func=pin_types.INPUT,unit=1),
            Pin(num='7',name='GND',func=pin_types.PWRIN,unit=1),
            Pin(num='8',name='B_NAND_OUTPUT',func=pin_types.INPUT,unit=1),
            Pin(num='9',name='B_AND_OUTPUT',func=pin_types.INPUT,unit=1)], 'unit_defs':[] }),
        Part(**{ 'name':'UA9638CDE4', 'dest':TEMPLATE, 'tool':SKIDL, 'aliases':Alias({'UA9638CDE4'}), 'ref_prefix':'U', 'fplist':['Package_SO:SOIC-8_3.9x4.9mm_P1.27mm', 'Package_SO:SOIC-8_3.9x4.9mm_P1.27mm'], 'footprint':'Package_SO:SOIC-8_3.9x4.9mm_P1.27mm', 'keywords':'Dual high-speed differential line driver', 'description':'', 'datasheet':'http://www.ti.com/lit/ds/symlink/ua9638.pdf', 'search_text':'/usr/share/kicad/symbols/Interface_LineDriver.kicad_sym\nUA9638CDE4\n\nDual high-speed differential line driver', 'pins':[
            Pin(num='1',name='VCC',func=pin_types.PWRIN,unit=1),
            Pin(num='2',name='IN',func=pin_types.INPUT,unit=1),
            Pin(num='4',name='GND',func=pin_types.PWRIN,unit=1),
            Pin(num='7',name='OUTA',func=pin_types.OUTPUT,unit=1),
            Pin(num='8',name='OUTB',func=pin_types.OUTPUT,unit=1),
            Pin(num='1',name='VCC',func=pin_types.PWRIN,unit=2),
            Pin(num='3',name='IN',func=pin_types.INPUT,unit=2),
            Pin(num='4',name='GND',func=pin_types.PWRIN,unit=2),
            Pin(num='5',name='OUTA',func=pin_types.OUTPUT,unit=2),
            Pin(num='6',name='OUTB',func=pin_types.OUTPUT,unit=2)], 'unit_defs':[{'label': 'uA', 'num': 1, 'pin_nums': ['1', '4', '8', '2', '7']},{'label': 'uB', 'num': 2, 'pin_nums': ['3', '5', '1', '6', '4']}] }),
        Part(**{ 'name':'UA9638CDG4', 'dest':TEMPLATE, 'tool':SKIDL, 'aliases':Alias({'UA9638CDG4'}), 'ref_prefix':'U', 'fplist':['Package_SO:SOIC-8_3.9x4.9mm_P1.27mm', 'Package_SO:SOIC-8_3.9x4.9mm_P1.27mm', 'Package_SO:SOIC-8_3.9x4.9mm_P1.27mm'], 'footprint':'Package_SO:SOIC-8_3.9x4.9mm_P1.27mm', 'keywords':'Dual high-speed differential line driver', 'description':'', 'datasheet':'http://www.ti.com/lit/ds/symlink/ua9638.pdf', 'search_text':'/usr/share/kicad/symbols/Interface_LineDriver.kicad_sym\nUA9638CDG4\n\nDual high-speed differential line driver', 'pins':[
            Pin(num='1',name='VCC',func=pin_types.PWRIN,unit=1),
            Pin(num='2',name='IN',func=pin_types.INPUT,unit=1),
            Pin(num='4',name='GND',func=pin_types.PWRIN,unit=1),
            Pin(num='7',name='OUTA',func=pin_types.OUTPUT,unit=1),
            Pin(num='8',name='OUTB',func=pin_types.OUTPUT,unit=1),
            Pin(num='1',name='VCC',func=pin_types.PWRIN,unit=2),
            Pin(num='3',name='IN',func=pin_types.INPUT,unit=2),
            Pin(num='4',name='GND',func=pin_types.PWRIN,unit=2),
            Pin(num='5',name='OUTA',func=pin_types.OUTPUT,unit=2),
            Pin(num='6',name='OUTB',func=pin_types.OUTPUT,unit=2)], 'unit_defs':[{'label': 'uA', 'num': 1, 'pin_nums': ['2', '7', '1', '4', '8']},{'label': 'uB', 'num': 2, 'pin_nums': ['5', '3', '6', '1', '4']}] }),
        Part(**{ 'name':'UA9638CDR', 'dest':TEMPLATE, 'tool':SKIDL, 'aliases':Alias({'UA9638CDR'}), 'ref_prefix':'U', 'fplist':['Package_SO:SOIC-8_3.9x4.9mm_P1.27mm', 'Package_SO:SOIC-8_3.9x4.9mm_P1.27mm', 'Package_SO:SOIC-8_3.9x4.9mm_P1.27mm', 'Package_SO:SOIC-8_3.9x4.9mm_P1.27mm'], 'footprint':'Package_SO:SOIC-8_3.9x4.9mm_P1.27mm', 'keywords':'Dual high-speed differential line driver', 'description':'', 'datasheet':'http://www.ti.com/lit/ds/symlink/ua9638.pdf', 'search_text':'/usr/share/kicad/symbols/Interface_LineDriver.kicad_sym\nUA9638CDR\n\nDual high-speed differential line driver', 'pins':[
            Pin(num='1',name='VCC',func=pin_types.PWRIN,unit=1),
            Pin(num='2',name='IN',func=pin_types.INPUT,unit=1),
            Pin(num='4',name='GND',func=pin_types.PWRIN,unit=1),
            Pin(num='7',name='OUTA',func=pin_types.OUTPUT,unit=1),
            Pin(num='8',name='OUTB',func=pin_types.OUTPUT,unit=1),
            Pin(num='1',name='VCC',func=pin_types.PWRIN,unit=2),
            Pin(num='3',name='IN',func=pin_types.INPUT,unit=2),
            Pin(num='4',name='GND',func=pin_types.PWRIN,unit=2),
            Pin(num='5',name='OUTA',func=pin_types.OUTPUT,unit=2),
            Pin(num='6',name='OUTB',func=pin_types.OUTPUT,unit=2)], 'unit_defs':[{'label': 'uA', 'num': 1, 'pin_nums': ['1', '8', '4', '2', '7']},{'label': 'uB', 'num': 2, 'pin_nums': ['1', '3', '5', '6', '4']}] }),
        Part(**{ 'name':'UA9638CDRG4', 'dest':TEMPLATE, 'tool':SKIDL, 'aliases':Alias({'UA9638CDRG4'}), 'ref_prefix':'U', 'fplist':['Package_SO:SOIC-8_3.9x4.9mm_P1.27mm', 'Package_SO:SOIC-8_3.9x4.9mm_P1.27mm', 'Package_SO:SOIC-8_3.9x4.9mm_P1.27mm', 'Package_SO:SOIC-8_3.9x4.9mm_P1.27mm', 'Package_SO:SOIC-8_3.9x4.9mm_P1.27mm'], 'footprint':'Package_SO:SOIC-8_3.9x4.9mm_P1.27mm', 'keywords':'Dual high-speed differential line driver', 'description':'', 'datasheet':'http://www.ti.com/lit/ds/symlink/ua9638.pdf', 'search_text':'/usr/share/kicad/symbols/Interface_LineDriver.kicad_sym\nUA9638CDRG4\n\nDual high-speed differential line driver', 'pins':[
            Pin(num='1',name='VCC',func=pin_types.PWRIN,unit=1),
            Pin(num='2',name='IN',func=pin_types.INPUT,unit=1),
            Pin(num='4',name='GND',func=pin_types.PWRIN,unit=1),
            Pin(num='7',name='OUTA',func=pin_types.OUTPUT,unit=1),
            Pin(num='8',name='OUTB',func=pin_types.OUTPUT,unit=1),
            Pin(num='1',name='VCC',func=pin_types.PWRIN,unit=2),
            Pin(num='3',name='IN',func=pin_types.INPUT,unit=2),
            Pin(num='4',name='GND',func=pin_types.PWRIN,unit=2),
            Pin(num='5',name='OUTA',func=pin_types.OUTPUT,unit=2),
            Pin(num='6',name='OUTB',func=pin_types.OUTPUT,unit=2)], 'unit_defs':[{'label': 'uA', 'num': 1, 'pin_nums': ['1', '2', '4', '7', '8']},{'label': 'uB', 'num': 2, 'pin_nums': ['3', '5', '4', '1', '6']}] }),
        Part(**{ 'name':'UA9638CPE4', 'dest':TEMPLATE, 'tool':SKIDL, 'aliases':Alias({'UA9638CPE4'}), 'ref_prefix':'U', 'fplist':['Package_DIP:DIP-8_W7.62mm', 'Package_DIP:DIP-8_W7.62mm'], 'footprint':'Package_DIP:DIP-8_W7.62mm', 'keywords':'Dual high-speed differential line driver', 'description':'', 'datasheet':'http://www.ti.com/lit/ds/symlink/ua9638.pdf', 'search_text':'/usr/share/kicad/symbols/Interface_LineDriver.kicad_sym\nUA9638CPE4\n\nDual high-speed differential line driver', 'pins':[
            Pin(num='1',name='VCC',func=pin_types.PWRIN,unit=1),
            Pin(num='2',name='IN',func=pin_types.INPUT,unit=1),
            Pin(num='4',name='GND',func=pin_types.PWRIN,unit=1),
            Pin(num='7',name='OUTA',func=pin_types.OUTPUT,unit=1),
            Pin(num='8',name='OUTB',func=pin_types.OUTPUT,unit=1),
            Pin(num='1',name='VCC',func=pin_types.PWRIN,unit=2),
            Pin(num='3',name='IN',func=pin_types.INPUT,unit=2),
            Pin(num='4',name='GND',func=pin_types.PWRIN,unit=2),
            Pin(num='5',name='OUTA',func=pin_types.OUTPUT,unit=2),
            Pin(num='6',name='OUTB',func=pin_types.OUTPUT,unit=2)], 'unit_defs':[{'label': 'uA', 'num': 1, 'pin_nums': ['1', '2', '4', '8', '7']},{'label': 'uB', 'num': 2, 'pin_nums': ['6', '1', '4', '3', '5']}] })])


from collections import defaultdict
from skidl import Pin, Part, Alias, SchLib, SKIDL, TEMPLATE

from skidl.pin import pin_types

SKIDL_lib_version = '0.0.1'

Reference_Current = SchLib(tool=SKIDL).add_parts(*[
        Part(**{ 'name':'LM134H', 'dest':TEMPLATE, 'tool':SKIDL, 'aliases':Alias({'LM134H'}), 'ref_prefix':'U', 'fplist':['Package_TO_SOT_THT:TO-46-3'], 'footprint':'Package_TO_SOT_THT:TO-46-3', 'keywords':'Adjustable Current Source 10mA', 'description':'', 'datasheet':'http://www.ti.com/lit/ds/symlink/lm134.pdf', 'search_text':'/usr/share/kicad/symbols/Reference_Current.kicad_sym\nLM134H\n\nAdjustable Current Source 10mA', 'pins':[
            Pin(num='1',name='~',func=pin_types.PASSIVE,unit=1),
            Pin(num='2',name='~',func=pin_types.PASSIVE,unit=1),
            Pin(num='3',name='~',func=pin_types.PASSIVE,unit=1)], 'unit_defs':[] }),
        Part(**{ 'name':'LM334M', 'dest':TEMPLATE, 'tool':SKIDL, 'aliases':Alias({'LM334M'}), 'ref_prefix':'U', 'fplist':['Package_SO:SOIC-8_3.9x4.9mm_P1.27mm'], 'footprint':'Package_SO:SOIC-8_3.9x4.9mm_P1.27mm', 'keywords':'Adjustable Current Source 10mA', 'description':'', 'datasheet':'http://www.ti.com/lit/ds/symlink/lm134.pdf', 'search_text':'/usr/share/kicad/symbols/Reference_Current.kicad_sym\nLM334M\n\nAdjustable Current Source 10mA', 'pins':[
            Pin(num='1',name='~',func=pin_types.PASSIVE,unit=1),
            Pin(num='2',name='~',func=pin_types.PASSIVE,unit=1),
            Pin(num='3',name='~',func=pin_types.PASSIVE,unit=1),
            Pin(num='4',name='~',func=pin_types.PASSIVE,unit=1),
            Pin(num='6',name='~',func=pin_types.PASSIVE,unit=1),
            Pin(num='7',name='~',func=pin_types.PASSIVE,unit=1)], 'unit_defs':[] }),
        Part(**{ 'name':'LM334SM', 'dest':TEMPLATE, 'tool':SKIDL, 'aliases':Alias({'LM334SM'}), 'ref_prefix':'U', 'fplist':['Package_SO:SOIC-8_3.9x4.9mm_P1.27mm'], 'footprint':'Package_SO:SOIC-8_3.9x4.9mm_P1.27mm', 'keywords':'Adjustable Current Source 10mA', 'description':'', 'datasheet':'http://www.ti.com/lit/ds/symlink/lm134.pdf', 'search_text':'/usr/share/kicad/symbols/Reference_Current.kicad_sym\nLM334SM\n\nAdjustable Current Source 10mA', 'pins':[
            Pin(num='1',name='~',func=pin_types.PASSIVE,unit=1),
            Pin(num='2',name='~',func=pin_types.PASSIVE,unit=1),
            Pin(num='3',name='~',func=pin_types.PASSIVE,unit=1)], 'unit_defs':[] }),
        Part(**{ 'name':'LM334Z', 'dest':TEMPLATE, 'tool':SKIDL, 'aliases':Alias({'LM334Z'}), 'ref_prefix':'U', 'fplist':['Package_TO_SOT_THT:TO-92_Inline'], 'footprint':'Package_TO_SOT_THT:TO-92_Inline', 'keywords':'Adjustable Current Source 10mA', 'description':'', 'datasheet':'http://www.ti.com/lit/ds/symlink/lm134.pdf', 'search_text':'/usr/share/kicad/symbols/Reference_Current.kicad_sym\nLM334Z\n\nAdjustable Current Source 10mA', 'pins':[
            Pin(num='1',name='~',func=pin_types.PASSIVE,unit=1),
            Pin(num='2',name='~',func=pin_types.PASSIVE,unit=1),
            Pin(num='3',name='~',func=pin_types.PASSIVE,unit=1)], 'unit_defs':[] }),
        Part(**{ 'name':'LT3092xDD', 'dest':TEMPLATE, 'tool':SKIDL, 'aliases':Alias({'LT3092xDD'}), 'ref_prefix':'U', 'fplist':['Package_DFN_QFN:DFN-8-1EP_3x3mm_P0.5mm_EP1.66x2.38mm'], 'footprint':'Package_DFN_QFN:DFN-8-1EP_3x3mm_P0.5mm_EP1.66x2.38mm', 'keywords':'current source', 'description':'', 'datasheet':'https://www.analog.com/media/en/technical-documentation/data-sheets/3092fc.pdf', 'search_text':'/usr/share/kicad/symbols/Reference_Current.kicad_sym\nLT3092xDD\n\ncurrent source', 'pins':[
            Pin(num='1',name='~',func=pin_types.PASSIVE,unit=1),
            Pin(num='2',name='~',func=pin_types.PASSIVE,unit=1),
            Pin(num='3',name='NC',func=pin_types.NOCONNECT,unit=1),
            Pin(num='4',name='~',func=pin_types.PASSIVE,unit=1),
            Pin(num='5',name='NC',func=pin_types.NOCONNECT,unit=1),
            Pin(num='6',name='NC',func=pin_types.NOCONNECT,unit=1),
            Pin(num='7',name='~',func=pin_types.PASSIVE,unit=1),
            Pin(num='8',name='~',func=pin_types.PASSIVE,unit=1),
            Pin(num='9',name='~',func=pin_types.PASSIVE,unit=1)], 'unit_defs':[] }),
        Part(**{ 'name':'LT3092xST', 'dest':TEMPLATE, 'tool':SKIDL, 'aliases':Alias({'LT3092xST'}), 'ref_prefix':'U', 'fplist':['Package_TO_SOT_SMD:SOT-223-3_TabPin2'], 'footprint':'Package_TO_SOT_SMD:SOT-223-3_TabPin2', 'keywords':'current source', 'description':'', 'datasheet':'https://www.analog.com/media/en/technical-documentation/data-sheets/3092fc.pdf', 'search_text':'/usr/share/kicad/symbols/Reference_Current.kicad_sym\nLT3092xST\n\ncurrent source', 'pins':[
            Pin(num='1',name='~',func=pin_types.PASSIVE,unit=1),
            Pin(num='2',name='~',func=pin_types.PASSIVE,unit=1),
            Pin(num='3',name='~',func=pin_types.PASSIVE,unit=1)], 'unit_defs':[] }),
        Part(**{ 'name':'LT3092xTS8', 'dest':TEMPLATE, 'tool':SKIDL, 'aliases':Alias({'LT3092xTS8'}), 'ref_prefix':'U', 'fplist':['Package_TO_SOT_SMD:TSOT-23-8'], 'footprint':'Package_TO_SOT_SMD:TSOT-23-8', 'keywords':'current source', 'description':'', 'datasheet':'https://www.analog.com/media/en/technical-documentation/data-sheets/3092fc.pdf', 'search_text':'/usr/share/kicad/symbols/Reference_Current.kicad_sym\nLT3092xTS8\n\ncurrent source', 'pins':[
            Pin(num='1',name='NC',func=pin_types.NOCONNECT,unit=1),
            Pin(num='2',name='~',func=pin_types.PASSIVE,unit=1),
            Pin(num='3',name='~',func=pin_types.PASSIVE,unit=1),
            Pin(num='4',name='~',func=pin_types.PASSIVE,unit=1),
            Pin(num='5',name='~',func=pin_types.PASSIVE,unit=1),
            Pin(num='6',name='NC',func=pin_types.NOCONNECT,unit=1),
            Pin(num='7',name='~',func=pin_types.PASSIVE,unit=1),
            Pin(num='8',name='~',func=pin_types.PASSIVE,unit=1)], 'unit_defs':[] }),
        Part(**{ 'name':'PSSI2021SAY', 'dest':TEMPLATE, 'tool':SKIDL, 'aliases':Alias({'PSSI2021SAY'}), 'ref_prefix':'U', 'fplist':['Package_TO_SOT_SMD:SOT-353_SC-70-5'], 'footprint':'Package_TO_SOT_SMD:SOT-353_SC-70-5', 'keywords':'iref adjustable', 'description':'', 'datasheet':'https://assets.nexperia.com/documents/data-sheet/PSSI2021SAY.pdf', 'search_text':'/usr/share/kicad/symbols/Reference_Current.kicad_sym\nPSSI2021SAY\n\niref adjustable', 'pins':[
            Pin(num='2',name='~',func=pin_types.PWROUT),
            Pin(num='1',name='NC',func=pin_types.NOCONNECT,unit=1),
            Pin(num='3',name='GND',func=pin_types.PWRIN,unit=1),
            Pin(num='4',name='~',func=pin_types.PASSIVE,unit=1),
            Pin(num='5',name='~',func=pin_types.PWRIN,unit=1)], 'unit_defs':[] }),
        Part(**{ 'name':'REF200AU', 'dest':TEMPLATE, 'tool':SKIDL, 'aliases':Alias({'REF200AU'}), 'ref_prefix':'U', 'fplist':['Package_SO:SOIC-8_3.9x4.9mm_P1.27mm'], 'footprint':'Package_SO:SOIC-8_3.9x4.9mm_P1.27mm', 'keywords':'Current Source/Sink 100Î¼A', 'description':'', 'datasheet':'www.ti.com/lit/ds/symlink/ref200.pdf', 'search_text':'/usr/share/kicad/symbols/Reference_Current.kicad_sym\nREF200AU\n\nCurrent Source/Sink 100Î¼A', 'pins':[
            Pin(num='6',name='S',func=pin_types.PASSIVE),
            Pin(num='1',name='~',func=pin_types.PASSIVE,unit=1),
            Pin(num='8',name='~',func=pin_types.PASSIVE,unit=1),
            Pin(num='2',name='~',func=pin_types.PASSIVE,unit=2),
            Pin(num='7',name='~',func=pin_types.PASSIVE,unit=2),
            Pin(num='3',name='~',func=pin_types.PASSIVE,unit=3),
            Pin(num='4',name='~',func=pin_types.PASSIVE,unit=3),
            Pin(num='5',name='~',func=pin_types.PASSIVE,unit=3)], 'unit_defs':[{'label': 'uA', 'num': 1, 'pin_nums': ['8', '6', '1']},{'label': 'uB', 'num': 2, 'pin_nums': ['2', '6', '7']},{'label': 'uC', 'num': 3, 'pin_nums': ['4', '5', '3', '6']}] }),
        Part(**{ 'name':'LM234Z-3', 'dest':TEMPLATE, 'tool':SKIDL, 'aliases':Alias({'LM234Z-3'}), 'ref_prefix':'U', 'fplist':['Package_TO_SOT_THT:TO-92_Inline', 'Package_TO_SOT_THT:TO-92_Inline'], 'footprint':'Package_TO_SOT_THT:TO-92_Inline', 'keywords':'Adjustable Current Source 10mA', 'description':'', 'datasheet':'http://www.ti.com/lit/ds/symlink/lm134.pdf', 'search_text':'/usr/share/kicad/symbols/Reference_Current.kicad_sym\nLM234Z-3\n\nAdjustable Current Source 10mA', 'pins':[
            Pin(num='1',name='~',func=pin_types.PASSIVE,unit=1),
            Pin(num='2',name='~',func=pin_types.PASSIVE,unit=1),
            Pin(num='3',name='~',func=pin_types.PASSIVE,unit=1)], 'unit_defs':[] }),
        Part(**{ 'name':'LM234Z-6', 'dest':TEMPLATE, 'tool':SKIDL, 'aliases':Alias({'LM234Z-6'}), 'ref_prefix':'U', 'fplist':['Package_TO_SOT_THT:TO-92_Inline', 'Package_TO_SOT_THT:TO-92_Inline', 'Package_TO_SOT_THT:TO-92_Inline'], 'footprint':'Package_TO_SOT_THT:TO-92_Inline', 'keywords':'Adjustable Current Source 10mA', 'description':'', 'datasheet':'http://www.ti.com/lit/ds/symlink/lm134.pdf', 'search_text':'/usr/share/kicad/symbols/Reference_Current.kicad_sym\nLM234Z-6\n\nAdjustable Current Source 10mA', 'pins':[
            Pin(num='1',name='~',func=pin_types.PASSIVE,unit=1),
            Pin(num='2',name='~',func=pin_types.PASSIVE,unit=1),
            Pin(num='3',name='~',func=pin_types.PASSIVE,unit=1)], 'unit_defs':[] }),
        Part(**{ 'name':'LM334Z-LFT1', 'dest':TEMPLATE, 'tool':SKIDL, 'aliases':Alias({'LM334Z-LFT1'}), 'ref_prefix':'U', 'fplist':['Package_TO_SOT_THT:TO-92_Inline', 'Package_TO_SOT_THT:TO-92_Inline', 'Package_TO_SOT_THT:TO-92_Inline', 'Package_TO_SOT_THT:TO-92_Inline'], 'footprint':'Package_TO_SOT_THT:TO-92_Inline', 'keywords':'Adjustable Current Source 10mA', 'description':'', 'datasheet':'http://www.ti.com/lit/ds/symlink/lm134.pdf', 'search_text':'/usr/share/kicad/symbols/Reference_Current.kicad_sym\nLM334Z-LFT1\n\nAdjustable Current Source 10mA', 'pins':[
            Pin(num='1',name='~',func=pin_types.PASSIVE,unit=1),
            Pin(num='2',name='~',func=pin_types.PASSIVE,unit=1),
            Pin(num='3',name='~',func=pin_types.PASSIVE,unit=1)], 'unit_defs':[] })])


# -*- coding: utf-8 -*-

# The MIT License (MIT) - Copyright (c) Dave Vandenbout.

"""
Functions for generating SVG.
"""

import math
from collections import namedtuple

from skidl.geometry import Tx, Point, BBox, tx_flip_y
from skidl.utilities import export_to_all


def draw_cmd_to_dict(symbol):
    """
    Convert a list of symbols from a KICAD part definition into a
    dictionary for easier access to properties.
    """
    d = {}
    name = symbol[0]
    items = symbol[1:]
    d = {}
    is_named_present = False
    item_names = []
    for item in items:
        # If the object is a list, recursively convert to dict
        if isinstance(item, list):
            item_name, item_dict = draw_cmd_to_dict(item)
            is_named_present = True
        # If the object is unnamed, put it in the "misc" list
        # ["key", item1, item2, ["xy", 0, 0]] -> "key": {"misc":[item1, item2], "xy":[0,0]
        else:
            item_name = "misc"
            item_dict = item

        # Multiple items with the same key (e.g. ("xy" 0 0) ("xy" 1 0))
        # get put into a list {"xy": [[0,0],[1,0]]}
        if item_name not in item_names:
            item_names.append(item_name)
        if item_name not in d:
            d[item_name] = [item_dict]
        else:
            d[item_name].append(item_dict)

    # if a list has only one item, remove it from the list
    for item_name in item_names:
        if len(d[item_name]) == 1:
            d[item_name] = d[item_name][0]

    if not is_named_present:
        d = d["misc"]

    return name, d


def bbox_to_svg(bbox, stroke_wid):
    return " ".join(
        [
            "<rect",
            'x="{bbox.min.x:.3f}" y="{bbox.min.y:.3f}"',
            'width="{bbox.w:.3f}" height="{bbox.h:.3f}"',
            'style="stroke-width:{stroke_wid:.3f}; stroke:#606060"',
            'class="$cell_id symbol"',
            "/>",
            "\n",
        ]
    ).format(**locals())


def draw_cmd_to_svg(draw_cmd, tx, part, net_stubs, max_stub_len):
    """Convert symbol drawing command into SVG string and an associated bounding box.

    Args:
        draw_cmd (str): Contains textual information about the shape to be drawn.
        tx (Tx): Transformation matrix to be applied to the shape.
        part (Part): Part object that the drawing command belongs to (used to get pin information.)
        net_stubs (list): List of Net objects whose names will be connected to part symbol pins as connection stubs.
        max_stub_len (int): Maximum length of a net stub name.

    Returns:
        shape_svg (str): SVG command for the shape.
        shape_bbox (BBox): Bounding box for the shape.
    """

    # Use this when determining width of a text string based on its number of characters.
    char_sz_fudge_factor = 0.6

    tx_scale = tx.scale  # Compute this only once.

    def points_to_str(*points):
        pt2str = lambda pt: f"{pt.x:.3f},{pt.y:.3f}"
        return " ".join((pt2str(pt) for pt in points))

    def pin_side(vec):
        """Determine the side of the symbol based on the pin's direction vector."""
        if vec.x > vec.y and vec.x > -vec.y:
            return "left"
        elif vec.x < vec.y and vec.x > -vec.y:
            return "top"
        elif vec.x < vec.y and vec.x < -vec.y:
            return "right"
        elif vec.x > vec.y and vec.x < -vec.y:
            return "bottom"
        else:
            raise RuntimeError("Impossible pin orientation.")

    def pin_text_to_svg(text, pin_attr, side, pt, char_wid):
        spc = "&#8201;&#8201;"  # Whitespaces for padding around pin text.
        svg_template = {
            "left": {
                "pin_name": '<text class="pin_name_text" x="{x:.3f}" y="{y:.3f}" transform="rotate(0 {x:.3f} {y:.3f})" style="font-size:{char_wid:.3f}" dominant-baseline="central" text-anchor="start">{spc}{text}</text>\n',
                "pin_num": '<text class="pin_num_text"  x="{x:.3f}" y="{y:.3f}" transform="rotate(0 {x:.3f} {y:.3f})" style="font-size:{char_wid:.3f}" dominant-baseline=""        text-anchor="end">{text}{spc}</text>\n',
                "net_name": '<text class="net_name_text" x="{x:.3f}" y="{y:.3f}" transform="rotate(0 {x:.3f} {y:.3f})" style="font-size:{char_wid:.3f}" dominant-baseline="central" text-anchor="end">{text}{spc}</text>\n',
            },
            "right": {
                "pin_name": '<text class="pin_name_text" x="{x:.3f}" y="{y:.3f}" transform="rotate(0 {x:.3f} {y:.3f})" style="font-size:{char_wid:.3f}" dominant-baseline="central" text-anchor="end">{text}{spc}</text>\n',
                "pin_num": '<text class="pin_num_text"  x="{x:.3f}" y="{y:.3f}" transform="rotate(0 {x:.3f} {y:.3f})" style="font-size:{char_wid:.3f}" dominant-baseline=""        text-anchor="start">{spc}{text}</text>\n',
                "net_name": '<text class="net_name_text" x="{x:.3f}" y="{y:.3f}" transform="rotate(0 {x:.3f} {y:.3f})" style="font-size:{char_wid:.3f}" dominant-baseline="central" text-anchor="start">{spc}{text}</text>\n',
            },
            "top": {
                "pin_name": '<text class="pin_name_text" x="{x:.3f}" y="{y:.3f}" transform="rotate(-90 {x:.3f} {y:.3f})" style="font-size:{char_wid:.3f}" dominant-baseline="central" text-anchor="end">{text}{spc}</text>\n',
                "pin_num": '<text class="pin_num_text"  x="{x:.3f}" y="{y:.3f}" transform="rotate(-90 {x:.3f} {y:.3f})" style="font-size:{char_wid:.3f}" dominant-baseline=""        text-anchor="start">{spc}{text}</text>\n',
                "net_name": '<text class="net_name_text" x="{x:.3f}" y="{y:.3f}" transform="rotate(-90 {x:.3f} {y:.3f})" style="font-size:{char_wid:.3f}" dominant-baseline="central" text-anchor="start">{spc}{text}</text>\n',
            },
            "bottom": {
                "pin_name": '<text class="pin_name_text" x="{x:.3f}" y="{y:.3f}" transform="rotate(-90 {x:.3f} {y:.3f})" style="font-size:{char_wid:.3f}" dominant-baseline="central" text-anchor="start">{spc}{text}</text>\n',
                "pin_num": '<text class="pin_num_text"  x="{x:.3f}" y="{y:.3f}" transform="rotate(-90 {x:.3f} {y:.3f})" style="font-size:{char_wid:.3f}" dominant-baseline=""        text-anchor="end">{text}{spc}</text>\n',
                "net_name": '<text class="net_name_text" x="{x:.3f}" y="{y:.3f}" transform="rotate(-90 {x:.3f} {y:.3f})" style="font-size:{char_wid:.3f}" dominant-baseline="central" text-anchor="end">{text}{spc}</text>\n',
            },
        }
        return svg_template[side][pin_attr].format(
            x=pt.x, y=pt.y, char_wid=char_wid, text=text, spc=spc
        )

    def text_to_svg(text, side, pt, char_wid, class_, attr):
        svg_template = {
            "right": '<text class="{class_}" x="{x:.3f}" y="{y:.3f}" transform="rotate(0 {x:.3f} {y:.3f})"   style="font-size:{char_wid:.3f}" dominant-baseline="central" text-anchor="end"   {attr}>{text}</text>\n',
            "left": '<text class="{class_}" x="{x:.3f}" y="{y:.3f}" transform="rotate(0 {x:.3f} {y:.3f})"   style="font-size:{char_wid:.3f}" dominant-baseline="central" text-anchor="start" {attr}>{text}</text>\n',
            "bottom": '<text class="{class_}" x="{x:.3f}" y="{y:.3f}" transform="rotate(-90 {x:.3f} {y:.3f})" style="font-size:{char_wid:.3f}" dominant-baseline="central" text-anchor="start" {attr}>{text}</text>\n',
            "top": '<text class="{class_}" x="{x:.3f}" y="{y:.3f}" transform="rotate(-90 {x:.3f} {y:.3f})" style="font-size:{char_wid:.3f}" dominant-baseline="central" text-anchor="end"   {attr}>{text}</text>\n',
        }
        return svg_template[side].format(
            x=pt.x, y=pt.y, char_wid=char_wid, text=text, class_=class_, attr=attr
        )

    def text_bbox(text, start, dir, char_wid, char_hgt):
        char_wid *= char_sz_fudge_factor  # Fudge-factor to make bbox turn out right.
        char_hgt *= char_sz_fudge_factor  # Fudge-factor to make bbox turn out right.
        ortho_dir = dir * Tx().rot(90)
        p1 = start - ortho_dir * char_hgt / 2
        p2 = start + ortho_dir * char_hgt / 2
        p3 = p1 + dir * char_wid * len(text)
        p4 = p2 + dir * char_wid * len(text)
        return BBox(p1, p2, p3, p4)

    shape_type, shape = draw_cmd_to_dict(draw_cmd)

    default_stroke_width = "1"
    default_stroke = "#000"

    if not "stroke" in shape:
        shape["stroke"] = {}
    if not "type" in shape["stroke"]:
        shape["stroke"]["type"] = "default"
    if not "width" in shape["stroke"]:
        shape["stroke"]["width"] = 0

    if not "fill" in shape:
        shape["fill"] = {}
    if not "type" in shape["fill"]:
        shape["fill"]["type"] = "none"
    if not "justify" in shape:
        shape["justify"] = "right"

    if shape["stroke"]["type"] == "default":
        shape["stroke"]["type"] = "#000"
    if shape["stroke"]["width"] == 0:
        shape["stroke"]["width"] = 0.1

    if shape_type == "polyline":
        points = [Point(*pt[0:2]) * tx for pt in shape["pts"]["xy"]]
        bbox = BBox(*points)
        points_str = points_to_str(*points)
        stroke = (shape["stroke"]["type"],)
        stroke_width = abs(shape["stroke"]["width"] * tx_scale)
        fill = shape["fill"]["type"]
        svg = " ".join(
            [
                "<polyline",
                'points="{points_str}"',
                'style="stroke-width:{stroke_width:.3f}"',
                'class="$cell_id symbol {fill}"',
                "/>",
            ]
        ).format(**locals())

    elif shape_type == "circle":
        ctr = Point(*shape["center"]) * tx
        radius = Point(shape["radius"], shape["radius"]) * tx_scale
        r = abs(radius.x)
        bbox = BBox(ctr + radius, ctr - radius)
        stroke = shape["stroke"]["type"]
        stroke_width = abs(shape["stroke"]["width"] * tx_scale)
        fill = shape["fill"]["type"]
        svg = " ".join(
            [
                "<circle",
                'cx="{ctr.x:.3f}" cy="{ctr.y:.3f}" r="{r:.3f}"',
                'style="stroke-width:{stroke_width:.3f}"',
                'class="$cell_id symbol {fill}"',
                "/>",
            ]
        ).format(**locals())

    elif shape_type == "rectangle":
        start = Point(*shape["start"]) * tx
        end = Point(*shape["end"]) * tx
        bbox = BBox(start, end)
        stroke = shape["stroke"]["type"]
        stroke_width = abs(shape["stroke"]["width"] * tx_scale)
        fill = shape["fill"]["type"]
        svg = " ".join(
            [
                "<rect",
                'x="{bbox.min.x:.3f}" y="{bbox.min.y:.3f}"',
                'width="{bbox.w:.3f}" height="{bbox.h:.3f}"',
                'style="stroke-width:{stroke_width:.3f}"',
                'class="$cell_id symbol {fill}"',
                "/>",
            ]
        ).format(**locals())

    elif shape_type == "arc":
        a = Point(*shape["start"]) * tx
        b = Point(*shape["end"]) * tx
        c = Point(*shape["mid"]) * tx
        bbox = BBox(a, b, c)

        A = (b - c).magnitude
        B = (a - c).magnitude
        C = (a - b).magnitude

        angle = math.acos((A * A + B * B - C * C) / (2 * A * B))
        K = 0.5 * A * B * math.sin(angle)
        r = A * B * C / 4 / K

        large_arc = int(math.pi / 2 > angle)
        sweep = int((b.x - a.x) * (c.y - a.y) - (b.y - a.y) * (c.x - a.x) < 0)
        stroke = shape["stroke"]["type"]
        stroke_width = abs(shape["stroke"]["width"] * tx_scale)
        fill = shape["fill"]["type"]
        svg = " ".join(
            [
                "<path",
                'd="M {a.x:.3f} {a.y:.3f} A {r:.3f} {r:.3f} 0 {large_arc} {sweep} {b.x:.3f} {b.y:.3f}"',
                'style="stroke-width:{stroke_width:.3f}"',
                'class="$cell_id symbol {fill}"',
                "/>",
            ]
        ).format(**locals())

    elif shape_type == "property":
        if "hide" in shape["effects"]:
            svg, bbox = "", BBox()
        else:
            if shape["misc"][0].lower() == "reference":
                class_ = "part_ref_text"
                attr = 's:attribute="ref"'
            elif shape["misc"][0].lower() == "value":
                class_ = "part_name_text"
                attr = 's:attribute="value"'
            else:
                raise RuntimeError(
                    f"Unknown property {symbol[1]} is not hidden."
                )
            start = Point(*shape["at"][0:2])
            rotation = shape["at"][2]
            justify = (
                shape["effects"].get("justify", shape.get("justify", "left")).lower()
            )
            dir = {"right": Point(-1, 0), "left": Point(1, 0)}[justify] * Tx().rot(
                rotation
            )
            end = start + dir
            start *= tx
            end *= tx
            side = pin_side(dir * tx.no_translate())
            char_wid, char_hgt = shape["effects"]["font"]["size"][:]
            char_wid *= tx_scale
            char_hgt *= tx_scale
            text = shape["misc"][1]
            svg = text_to_svg(text, side, start, char_wid, class_, attr)
            bbox = text_bbox(text, start, (end - start).norm, char_wid, char_hgt)

    elif shape_type == "pin":
        num_char_wid, num_char_hgt = shape["number"]["effects"]["font"]["size"][:]
        name_char_wid, name_char_hgt = shape["name"]["effects"]["font"]["size"][:]
        net_name_char_wid, net_name_char_hgt = name_char_wid, name_char_hgt

        # Get the pin object associated with this drawing command.
        pin_name = shape["name"]["misc"]
        pin_num = shape["number"]["misc"]
        pin = part[pin_num]
        if pin.net in [None, NC]:
            # Unconnected pins remain at the length of the default symbol pin.
            extension = 0
        elif pin.net in net_stubs:
            # Don't extend the pin since the net name for the stub will be
            # connected directly to the pin.
            extension = 0
        else:
            # The pin is connected to a non-stub (routed) net.
            # Extend the pin to the edge of the symbol bounding box so it
            # can be routed to.
            extension = net_name_char_wid * max_stub_len * char_sz_fudge_factor

        start = Point(*shape["at"][0:2])
        rotation = shape["at"][2]
        length = shape["length"]
        dir = Point(1, 0) * Tx().rot(rotation)
        end = start + dir * length
        start -= dir * extension
        end *= tx
        start *= tx
        side = pin_side(dir * tx.no_translate())

        # Bounding box for the pin.
        bbox = BBox(start, end)

        num_char_wid *= tx_scale
        num_char_hgt *= tx_scale
        name_char_wid *= tx_scale
        name_char_hgt *= tx_scale
        net_name_char_hgt *= tx_scale
        net_name_char_wid *= tx_scale

        # Now add bounding boxes for the pin number, name, and net name.
        bbox += text_bbox(
            pin_num, start, (end - start).norm, num_char_wid, num_char_hgt
        )
        bbox += text_bbox(
            pin_name, start, (end - start).norm, name_char_wid, name_char_hgt
        )
        if pin.net in net_stubs:
            bbox += text_bbox(
                pin.net.name,
                start,
                (start - end).norm,
                net_name_char_wid,
                net_name_char_hgt,
            )

        stroke = shape["stroke"]["type"]
        stroke_width = abs(shape["stroke"]["width"] * tx_scale)
        fill = shape["fill"]["type"]
        circle_stroke_width = 2 * stroke_width

        points_str = points_to_str(start, end)
        pin_svg = f'<polyline points="{points_str}" style="stroke-width:{stroke_width:.3f}" class="$cell_id symbol {fill}" />\n'
        # pin_circle_svg = f'<circle cx="{start.x:.3f}" cy="{start.y:.3f}" r="{circle_stroke_width:.3f}" style="stroke-width:{circle_stroke_width:.3f}" class="$cell_id symbol {fill}" />\n'
        pin_circle_svg = ""
        pin_num_svg = pin_text_to_svg(pin_num, "pin_num", side, end, num_char_wid)
        pin_name_svg = pin_text_to_svg(pin_name, "pin_name", side, end, name_char_wid)
        if pin.net in net_stubs:
            net_name_svg = pin_text_to_svg(
                pin.net.name, "net_name", side, start, net_name_char_wid
            )
        else:
            net_name_svg = ""
        connection_svg = f'<g s:x="{start.x:.3f}" s:y="{start.y:.3f}" s:pid="{pin_num}" s:position="{side}"/>\n'
        svg = "".join(
            [
                pin_svg,
                pin_circle_svg,
                pin_num_svg,
                pin_name_svg,
                net_name_svg,
                connection_svg,
            ]
        )

    elif shape_type == "text":
        class_ = "text"
        attr = ""
        start = Point(*shape["at"][0:2])
        rotation = shape["at"][2]
        dir = {"right": Point(1, 0), "left": Point(-1, 0)}[
            shape["justify"].lower()
        ] * Tx().rot(rotation)
        end = start + dir
        start *= tx
        end *= tx
        side = pin_side(dir * tx.no_translate())
        char_wid, char_hgt = shape["effects"]["font"]["size"][:]
        char_wid *= tx_scale
        char_hgt *= tx_scale
        text = shape["misc"]
        svg = text_to_svg(text, side, start, char_wid, class_, attr)
        bbox = text_bbox(text, start, (end - start).norm, char_wid, char_hgt)

    else:
        raise RuntimeError(f"Unrecognized shape type: {shape_type}")

    return svg, bbox


@export_to_all
def gen_svg_comp(part, symtx, net_stubs=None):
    """
    Generate SVG for this component.

    Args:
        part: Part object for which an SVG symbol will be created.
        net_stubs: List of Net objects whose names will be connected to
            part symbol pins as connection stubs.
        symtx: String such as "HR" that indicates symbol mirroring/rotation.

    Returns: SVG for the part symbol.
    """

    # Create transformation matrix for the symbol from symtx, flip Y axis, and scale.
    px = 96  # Pixels per inch. SVG uses pixels.
    mm = 25.4  # Millimeters per inch. KiCad uses millimeters.
    scale = px / mm  # Scale for converting KiCad units (mm) to SVG units (pixels).
    scale *= 2.54  # Adjustment for matching symbol sizes with netlistsvg's I/O ports.
    tx = Tx.from_symtx(symtx) * tx_flip_y * scale

    # Get maximum length of net stub name if any are needed for this part symbol.
    net_stubs = net_stubs or []  # Empty list of stub nets if argument is None.
    max_stub_len = 0  # If no net stubs are needed, this stays at zero.
    for pin in part:
        for net in pin.nets:
            # Don't let names for no-connect nets affect maximum stub length.
            if net in [NC, None]:
                continue
            if net in net_stubs:
                max_stub_len = max(len(net.name), max_stub_len)

    # Assemble and name the SVGs for all the part units.
    svg = []
    for unit in part.unit.values():

        # First, compute the bounding box of the part symbol.
        bbox = BBox()
        for cmd in part.draw_cmds[unit.num]:
            _, bb = draw_cmd_to_svg(cmd, tx, part, net_stubs, max_stub_len)
            bbox.add(bb)

        # Add translation to the transformation matrix so the part's bounding box
        # starts at (0,0). (netlistsvg seems to malfunction, otherwise.)
        trans_tx = tx.move(-bbox.min)

        # Finally, recalculate the part symbol with the added translation.
        bbox = BBox()
        unit_svg = []
        for cmd in part.draw_cmds[unit.num]:
            s, bb = draw_cmd_to_svg(cmd, trans_tx, part, net_stubs, max_stub_len)
            bbox.add(bb)
            unit_svg.append(s)

        # Assign part unit name.
        if max_stub_len:
            # If net stubs are attached to symbol, then it's only to be used
            # for a specific part. Therefore, tag the symbol name with the unique
            # part reference so it will only be used by this part.
            symbol_name = f"{part.name}_{part.ref}_{unit.num}_{symtx}"
        else:
            # No net stubs means this symbol can be used for any part that
            # also has no net stubs, so don't tag it with a specific part reference.
            symbol_name = f"{part.name}_{unit.num}_{symtx}"

        # Begin SVG for part unit. Translate it so the bbox.min is at (0,0).
        translate = -bbox.min
        svg.append(
            " ".join(
                [
                    "<g",
                    's:type="{symbol_name}"',
                    's:width="{bbox.w:.3f}"',
                    's:height="{bbox.h:.3f}"',
                    'transform="translate({translate.x:.3f} {translate.y:.3f})"',
                    ">\n",
                ]
            ).format(**locals())
        )

        # Add part alias.
        svg.append(f'<s:alias val="{symbol_name}"/>\n')

        for item in unit_svg:
            if "text" not in item:
                svg.append(item)

        for item in unit_svg:
            if "text" in item:
                svg.append(item)

        # Place a visible bounding-box around symbol for trouble-shooting.
        show_bbox = False
        bbox_stroke_width = scale * 0.1
        if show_bbox:
            svg.append(bbox_to_svg(bbox, bbox_stroke_width))

        # Finish SVG for part unit.
        svg.append("</g>\n")

    return "".join(svg)



from collections import defaultdict
from skidl import Pin, Part, Alias, SchLib, SKIDL, TEMPLATE

from skidl.pin import pin_types

SKIDL_lib_version = '0.0.1'

MCU_ST_STM32WBA = SchLib(tool=SKIDL).add_parts(*[
        Part(**{ 'name':'STM32WBA52CEUx', 'dest':TEMPLATE, 'tool':SKIDL, 'aliases':Alias({'STM32WBA52CEUx'}), 'ref_prefix':'U', 'fplist':['Package_DFN_QFN:QFN-48-1EP_7x7mm_P0.5mm_EP5.6x5.6mm'], 'footprint':'Package_DFN_QFN:QFN-48-1EP_7x7mm_P0.5mm_EP5.6x5.6mm', 'keywords':'Arm Cortex-M33 STM32WBA STM32WBAx2', 'description':'', 'datasheet':'https://www.st.com/resource/en/datasheet/stm32wba52ce.pdf', 'search_text':'/usr/share/kicad/symbols/MCU_ST_STM32WBA.kicad_sym\nSTM32WBA52CEUx\n\nArm Cortex-M33 STM32WBA STM32WBAx2', 'pins':[
            Pin(num='1',name='PB12',func=pin_types.BIDIR,unit=1),
            Pin(num='10',name='PA2',func=pin_types.BIDIR,unit=1),
            Pin(num='11',name='VDD',func=pin_types.PWRIN,unit=1),
            Pin(num='12',name='PA1',func=pin_types.BIDIR,unit=1),
            Pin(num='13',name='PA0',func=pin_types.BIDIR,unit=1),
            Pin(num='14',name='PB9',func=pin_types.BIDIR,unit=1),
            Pin(num='15',name='PB8',func=pin_types.BIDIR,unit=1),
            Pin(num='16',name='PC15',func=pin_types.BIDIR,unit=1),
            Pin(num='17',name='PC14',func=pin_types.BIDIR,unit=1),
            Pin(num='18',name='PC13',func=pin_types.BIDIR,unit=1),
            Pin(num='19',name='PB7',func=pin_types.BIDIR,unit=1),
            Pin(num='2',name='PB11',func=pin_types.BIDIR,unit=1),
            Pin(num='20',name='PB6',func=pin_types.BIDIR,unit=1),
            Pin(num='21',name='PB5',func=pin_types.BIDIR,unit=1),
            Pin(num='22',name='VDD',func=pin_types.PWRIN,unit=1),
            Pin(num='23',name='PB4',func=pin_types.BIDIR,unit=1),
            Pin(num='24',name='PB3',func=pin_types.BIDIR,unit=1),
            Pin(num='25',name='PA15',func=pin_types.BIDIR,unit=1),
            Pin(num='26',name='PA14',func=pin_types.BIDIR,unit=1),
            Pin(num='27',name='PA13',func=pin_types.BIDIR,unit=1),
            Pin(num='28',name='PA12',func=pin_types.BIDIR,unit=1),
            Pin(num='29',name='PA11',func=pin_types.BIDIR,unit=1),
            Pin(num='3',name='PA8',func=pin_types.BIDIR,unit=1),
            Pin(num='30',name='PB2',func=pin_types.BIDIR,unit=1),
            Pin(num='31',name='PB1',func=pin_types.BIDIR,unit=1),
            Pin(num='32',name='PB0',func=pin_types.BIDIR,unit=1),
            Pin(num='33',name='PB15',func=pin_types.BIDIR,unit=1),
            Pin(num='34',name='PH3',func=pin_types.BIDIR,unit=1),
            Pin(num='35',name='VDD',func=pin_types.PWRIN,unit=1),
            Pin(num='36',name='NRST',func=pin_types.INPUT,unit=1),
            Pin(num='37',name='RF',func=pin_types.BIDIR,unit=1),
            Pin(num='38',name='VDDHPA',func=pin_types.PWRIN,unit=1),
            Pin(num='39',name='VDDRF',func=pin_types.PWRIN,unit=1),
            Pin(num='4',name='PA7',func=pin_types.BIDIR,unit=1),
            Pin(num='40',name='OSC_OUT',func=pin_types.INPUT,unit=1),
            Pin(num='41',name='OSC_IN',func=pin_types.INPUT,unit=1),
            Pin(num='42',name='VDDRFPA',func=pin_types.PWRIN,unit=1),
            Pin(num='43',name='VDD',func=pin_types.PWRIN,unit=1),
            Pin(num='44',name='VCAP',func=pin_types.PWROUT,unit=1),
            Pin(num='45',name='PA10',func=pin_types.BIDIR,unit=1),
            Pin(num='46',name='PA9',func=pin_types.BIDIR,unit=1),
            Pin(num='47',name='PB14',func=pin_types.BIDIR,unit=1),
            Pin(num='48',name='PB13',func=pin_types.BIDIR,unit=1),
            Pin(num='49',name='VSS/VSS',func=pin_types.PWRIN,unit=1),
            Pin(num='5',name='PA6',func=pin_types.BIDIR,unit=1),
            Pin(num='6',name='VDDA',func=pin_types.PWRIN,unit=1),
            Pin(num='7',name='PA5',func=pin_types.BIDIR,unit=1),
            Pin(num='8',name='PA3',func=pin_types.BIDIR,unit=1),
            Pin(num='9',name='PB10',func=pin_types.BIDIR,unit=1)], 'unit_defs':[] }),
        Part(**{ 'name':'STM32WBA52CGUx', 'dest':TEMPLATE, 'tool':SKIDL, 'aliases':Alias({'STM32WBA52CGUx'}), 'ref_prefix':'U', 'fplist':['Package_DFN_QFN:QFN-48-1EP_7x7mm_P0.5mm_EP5.6x5.6mm'], 'footprint':'Package_DFN_QFN:QFN-48-1EP_7x7mm_P0.5mm_EP5.6x5.6mm', 'keywords':'Arm Cortex-M33 STM32WBA STM32WBAx2', 'description':'', 'datasheet':'https://www.st.com/resource/en/datasheet/stm32wba52cg.pdf', 'search_text':'/usr/share/kicad/symbols/MCU_ST_STM32WBA.kicad_sym\nSTM32WBA52CGUx\n\nArm Cortex-M33 STM32WBA STM32WBAx2', 'pins':[
            Pin(num='1',name='PB12',func=pin_types.BIDIR,unit=1),
            Pin(num='10',name='PA2',func=pin_types.BIDIR,unit=1),
            Pin(num='11',name='VDD',func=pin_types.PWRIN,unit=1),
            Pin(num='12',name='PA1',func=pin_types.BIDIR,unit=1),
            Pin(num='13',name='PA0',func=pin_types.BIDIR,unit=1),
            Pin(num='14',name='PB9',func=pin_types.BIDIR,unit=1),
            Pin(num='15',name='PB8',func=pin_types.BIDIR,unit=1),
            Pin(num='16',name='PC15',func=pin_types.BIDIR,unit=1),
            Pin(num='17',name='PC14',func=pin_types.BIDIR,unit=1),
            Pin(num='18',name='PC13',func=pin_types.BIDIR,unit=1),
            Pin(num='19',name='PB7',func=pin_types.BIDIR,unit=1),
            Pin(num='2',name='PB11',func=pin_types.BIDIR,unit=1),
            Pin(num='20',name='PB6',func=pin_types.BIDIR,unit=1),
            Pin(num='21',name='PB5',func=pin_types.BIDIR,unit=1),
            Pin(num='22',name='VDD',func=pin_types.PWRIN,unit=1),
            Pin(num='23',name='PB4',func=pin_types.BIDIR,unit=1),
            Pin(num='24',name='PB3',func=pin_types.BIDIR,unit=1),
            Pin(num='25',name='PA15',func=pin_types.BIDIR,unit=1),
            Pin(num='26',name='PA14',func=pin_types.BIDIR,unit=1),
            Pin(num='27',name='PA13',func=pin_types.BIDIR,unit=1),
            Pin(num='28',name='PA12',func=pin_types.BIDIR,unit=1),
            Pin(num='29',name='PA11',func=pin_types.BIDIR,unit=1),
            Pin(num='3',name='PA8',func=pin_types.BIDIR,unit=1),
            Pin(num='30',name='PB2',func=pin_types.BIDIR,unit=1),
            Pin(num='31',name='PB1',func=pin_types.BIDIR,unit=1),
            Pin(num='32',name='PB0',func=pin_types.BIDIR,unit=1),
            Pin(num='33',name='PB15',func=pin_types.BIDIR,unit=1),
            Pin(num='34',name='PH3',func=pin_types.BIDIR,unit=1),
            Pin(num='35',name='VDD',func=pin_types.PWRIN,unit=1),
            Pin(num='36',name='NRST',func=pin_types.INPUT,unit=1),
            Pin(num='37',name='RF',func=pin_types.BIDIR,unit=1),
            Pin(num='38',name='VDDHPA',func=pin_types.PWRIN,unit=1),
            Pin(num='39',name='VDDRF',func=pin_types.PWRIN,unit=1),
            Pin(num='4',name='PA7',func=pin_types.BIDIR,unit=1),
            Pin(num='40',name='OSC_OUT',func=pin_types.INPUT,unit=1),
            Pin(num='41',name='OSC_IN',func=pin_types.INPUT,unit=1),
            Pin(num='42',name='VDDRFPA',func=pin_types.PWRIN,unit=1),
            Pin(num='43',name='VDD',func=pin_types.PWRIN,unit=1),
            Pin(num='44',name='VCAP',func=pin_types.PWROUT,unit=1),
            Pin(num='45',name='PA10',func=pin_types.BIDIR,unit=1),
            Pin(num='46',name='PA9',func=pin_types.BIDIR,unit=1),
            Pin(num='47',name='PB14',func=pin_types.BIDIR,unit=1),
            Pin(num='48',name='PB13',func=pin_types.BIDIR,unit=1),
            Pin(num='49',name='VSS/VSS',func=pin_types.PWRIN,unit=1),
            Pin(num='5',name='PA6',func=pin_types.BIDIR,unit=1),
            Pin(num='6',name='VDDA',func=pin_types.PWRIN,unit=1),
            Pin(num='7',name='PA5',func=pin_types.BIDIR,unit=1),
            Pin(num='8',name='PA3',func=pin_types.BIDIR,unit=1),
            Pin(num='9',name='PB10',func=pin_types.BIDIR,unit=1)], 'unit_defs':[] })])